// MDIEditorView.cpp : implementation of the CMDIEditorView class
//

#include "stdafx.h"
#include "MDIEditor.h"

#include "MDIEditorDoc.h"
#include "MDIEditorView.h"
#include "CustomEditCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMDIEditorView

IMPLEMENT_DYNCREATE(CMDIEditorView, CBCGPEditView)

BEGIN_MESSAGE_MAP(CMDIEditorView, CBCGPEditView)
	//{{AFX_MSG_MAP(CMDIEditorView)
	ON_WM_CREATE()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_EDIT_TOGGLEBOOKMARK, OnEditTogglebookmark)
	ON_COMMAND(ID_EDIT_NEXTBOOKMARK, OnEditNextbookmark)
	ON_COMMAND(ID_EDIT_PREVIOUSBOOKMARK, OnEditPreviousbookmark)
	ON_COMMAND(ID_EDIT_LISTMEMBERS, OnEditListmembers)
	ON_COMMAND(ID_EDIT_FINDNEXT, OnEditFindnext)
	ON_COMMAND(ID_EDIT_FINDPREV, OnEditFindprev)
	ON_COMMAND(ID_EDIT_TOGGLEBREAKPOINT, OnEditTogglebreakpoint)
	ON_UPDATE_COMMAND_UI(ID_EDIT_TOGGLEBREAKPOINT, OnUpdateEditTogglebreakpoint)
	ON_COMMAND(ID_EDIT_REMOVE_ALL_BREAKPOINTS, OnEditRemoveAllBreakpoints)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REMOVE_ALL_BREAKPOINTS, OnUpdateEditRemoveAllBreakpoints)
	ON_UPDATE_COMMAND_UI(ID_EDIT_LISTMEMBERS, OnUpdateEditListmembers)
	ON_COMMAND(ID_VIEW_FONT, OnViewFont)
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_CURSORPOS, OnUpdateCursorPos)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CBCGPEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CBCGPEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CBCGPEditView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMDIEditorView construction/destruction

CMDIEditorView::CMDIEditorView()
{
	m_pEdit = NULL;
}

CMDIEditorView::~CMDIEditorView()
{
}

BOOL CMDIEditorView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CBCGPEditView::PreCreateWindow(cs);
}

CBCGPEditCtrl* CMDIEditorView::CreateEdit ()
{
	m_pEdit = new CCustomEditCtrl;
	return m_pEdit;
}

/////////////////////////////////////////////////////////////////////////////
// CMDIEditorView drawing

void CMDIEditorView::OnDraw(CDC* pDC)
{
	CMDIEditorDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CMDIEditorView printing

BOOL CMDIEditorView::OnPreparePrinting(CPrintInfo* pInfo)
{
	return CBCGPEditView::OnPreparePrinting(pInfo);
}

void CMDIEditorView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	CBCGPEditView::OnBeginPrinting(pDC, pInfo);
	// TODO: add extra initialization before printing
}

void CMDIEditorView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	// TODO: add cleanup after printing
	CBCGPEditView::OnEndPrinting(pDC, pInfo);
}

/////////////////////////////////////////////////////////////////////////////
// CMDIEditorView diagnostics

#ifdef _DEBUG
void CMDIEditorView::AssertValid() const
{
	CBCGPEditView::AssertValid();
}

void CMDIEditorView::Dump(CDumpContext& dc) const
{
	CBCGPEditView::Dump(dc);
}

CMDIEditorDoc* CMDIEditorView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMDIEditorDoc)));
	return (CMDIEditorDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMDIEditorView message handlers

int CMDIEditorView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CBCGPEditView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Enabling tooltips, creating image lists, setting fonts 
	GetEditCtrl ()->EnableToolTips ();
	ResetDefaultFont();

	if (BOOL bImgCreate = m_imgList.Create (IDB_IMG_INTELLISENSE, 16, 0, RGB (255, 255, 255)))
	{
		GetEditCtrl ()->SetIntelliSenseImgList (&m_imgList);
	}
	else
	{
		ASSERT(FALSE);
	}

	return 0;
}

void CMDIEditorView::OnInitialUpdate() 
{
	CBCGPEditView::OnInitialUpdate();
	
	ASSERT_VALID (GetEditCtrl ());

	// Load XML settings for syntax highlighting
	CString strXMLFileName = GetDocument()->m_strXMLFileName;
	if (!strXMLFileName.IsEmpty ())
	{
		GetEditCtrl ()->LoadXMLSettings(strXMLFileName);
	}

	// Enable IntelliSense support 
	const CString& strPathName = GetDocument()->GetPathName();
	switch (GetDocument()->m_uiExampleID)
	{
	case ID_EXAMPLE_CPP:
		GetEditCtrl ()->EnableIntelliSense(strPathName.IsEmpty());
		m_pEdit->EnableBreakpoints(TRUE);
		m_pEdit->m_bCheckColorTags = FALSE;
		break;

	case ID_EXAMPLE_WEB:
		GetEditCtrl ()->EnableIntelliSense(FALSE);
		m_pEdit->EnableBreakpoints(FALSE);
		m_pEdit->m_bCheckColorTags = TRUE;

	default:
		GetEditCtrl ()->EnableIntelliSense(FALSE);
		m_pEdit->EnableBreakpoints(FALSE);
		m_pEdit->RemoveAllBreakpoints();
	}

	// Preset Markers
	if (strPathName.IsEmpty() && GetDocument()->m_uiExampleID == ID_EXAMPLE_CPP)
	{
		GetEditCtrl ()->ToggleMarker (3, g_dwBCGPEdit_BookMark);
		GetEditCtrl ()->ToggleMarker (11, g_dwBCGPEdit_BookMark);
		GetEditCtrl ()->ToggleMarker (20, g_dwBCGPEdit_BookMark);
		GetEditCtrl ()->ToggleMarker (30, g_dwBCGPEdit_BookMark);
		GetEditCtrl ()->ToggleMarker (45, g_dwBCGPEdit_BookMark);
		GetEditCtrl ()->ToggleMarker (48, g_dwBCGPEdit_BookMark);
	}
	else
	{
		GetEditCtrl ()->DeleteAllMarkers();
	}

	// for new documents
	if (strPathName.IsEmpty())
	{
		RestoreDefaultData();
	}
}

void CMDIEditorView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	ASSERT_VALID (GetEditCtrl ());

	CPoint pt(point);

	ScreenToClient(&pt);
	if (pt.x < GetEditCtrl ()->m_nLeftMarginWidth)
	{
		pt.x = GetEditCtrl ()->m_nLeftMarginWidth;
		GetEditCtrl ()->SetCaret(GetEditCtrl ()->HitTest(pt));

		CMenu menu;
		if (menu.LoadMenu(IDR_POPUP_SELECTIONMARGIN))
		{
			CMenu* pPopup = menu.GetSubMenu(0);
			ASSERT(pPopup != NULL);

			pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd());
		}
	}
	else
	{
		CMenu menu;
		if (menu.LoadMenu(IDR_CONTEXT_MENU))
		{
			CMenu* pPopup = menu.GetSubMenu(0);
			ASSERT(pPopup != NULL);

			pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd());
		}
	}
}

void CMDIEditorView::ResetDefaultFont()
{
	ASSERT_VALID (GetEditCtrl ());
	
	HFONT hFont = (HFONT)::GetStockObject(DEFAULT_GUI_FONT);
	CDC* pDC = GetDC();

	CFont* pFont = pDC->SelectObject(CFont::FromHandle(hFont));
	pDC->SelectObject(pFont);
	::DeleteObject(hFont);

	LOGFONT lf;
	pFont->GetLogFont (&lf);

	CString strFontName(_T("Courier New"));
	CopyMemory(lf.lfFaceName,(LPCTSTR)strFontName,(strFontName.GetLength() + 1) * sizeof(TCHAR));
	lf.lfWidth = 0;
	lf.lfEscapement = 0;
	lf.lfOrientation = 0;
	lf.lfWeight = FW_NORMAL;
	lf.lfItalic = 0;
	lf.lfUnderline = 0;
	lf.lfStrikeOut = 0;

	m_Font.CreateFontIndirect(&lf);
	GetEditCtrl ()->SetFont (&m_Font);
}

BOOL CMDIEditorView::RestoreDefaultData()
{
	ASSERT_VALID (GetEditCtrl ());

	UINT uiID = GetDocument()->m_uiExampleID;
	if (uiID == 0)
	{
		return FALSE;
	}

	BOOL bRet = FALSE;
	LPCTSTR lpszResourceName = MAKEINTRESOURCE (uiID);
	ASSERT(lpszResourceName != NULL);

	// Load example from resource
	HINSTANCE hInst = AfxFindResourceHandle(lpszResourceName, _T("Examples"));
	HRSRC hRsrc = ::FindResource(hInst, lpszResourceName, _T("Examples"));
	
	if (hRsrc == NULL)
	{
		ASSERT(FALSE);
	}
	else
	{
		HGLOBAL hGlobal = LoadResource(hInst, hRsrc);

		if (hGlobal == NULL)
		{
			ASSERT(FALSE);
		}
		else
		{
			LPSTR lpszText = (LPSTR)LockResource(hGlobal);

			if (lpszText == NULL)
			{
				ASSERT(FALSE);
			}
			else
			{
				USES_CONVERSION;
				GetEditCtrl ()->SetWindowText(A2T(lpszText));
				bRet = TRUE;
			}

			UnlockResource(hGlobal);
			FreeResource(hGlobal);
		}
	}

	return bRet;
}

void CMDIEditorView::OnEditTogglebookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->ToggleMarker (GetEditCtrl ()->GetCurRow (), g_dwBCGPEdit_BookMark);
}

void CMDIEditorView::OnEditNextbookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->GoToNextMarker(g_dwBCGPEdit_BookMark, TRUE);
}

void CMDIEditorView::OnEditPreviousbookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->GoToNextMarker(g_dwBCGPEdit_BookMark,FALSE);
}

void CMDIEditorView::OnEditListmembers() 
{
	ASSERT_VALID (m_pEdit);
	m_pEdit->InvokeIntelliSense();
}

void CMDIEditorView::OnUpdateEditListmembers(CCmdUI* pCmdUI) 
{
	ASSERT_VALID (m_pEdit);

	CObList lstIntelliSenseData;

	m_pEdit->FillIntelliSenseList(lstIntelliSenseData);
	pCmdUI->Enable(!lstIntelliSenseData.IsEmpty());
	
	m_pEdit->ReleaseIntelliSenseList(lstIntelliSenseData);
}

void CMDIEditorView::OnEditFindnext() 
{
	m_dwFindMask |= FR_DOWN;
	OnFindReplace(0, 0);
}

void CMDIEditorView::OnEditFindprev() 
{
	m_dwFindMask &= ~FR_DOWN;
	OnFindReplace(0, 0);
}

void CMDIEditorView::OnEditTogglebreakpoint() 
{
	ASSERT_VALID (m_pEdit);
	m_pEdit->ToggleBreakpoint();
}

void CMDIEditorView::OnEditRemoveAllBreakpoints() 
{
	ASSERT_VALID (m_pEdit);
	m_pEdit->RemoveAllBreakpoints();
}

void CMDIEditorView::OnUpdateEditTogglebreakpoint(CCmdUI* pCmdUI) 
{
	ASSERT(pCmdUI != NULL);
	pCmdUI->Enable (GetDocument()->m_uiExampleID == ID_EXAMPLE_CPP);
}

void CMDIEditorView::OnUpdateEditRemoveAllBreakpoints(CCmdUI* pCmdUI) 
{
	ASSERT(pCmdUI != NULL);
	pCmdUI->Enable (GetDocument()->m_uiExampleID == ID_EXAMPLE_CPP);
}

BOOL CMDIEditorView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	ASSERT_VALID (GetEditCtrl ());
	GetDocument()->SetModifiedFlag(GetEditCtrl ()->IsModified());

	return CBCGPEditView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CMDIEditorView::OnViewFont() 
{
	ASSERT_VALID (GetEditCtrl ());

	LOGFONT lf;
	GetEditCtrl ()->GetFont ()->GetLogFont (&lf);

	DWORD	dwFontDialogFlags = CF_EFFECTS | CF_SCREENFONTS;

	CFontDialog dlg (&lf, dwFontDialogFlags, NULL, this);
	dlg.m_cf.rgbColors = GetEditCtrl ()->m_clrText;
	if (dlg.DoModal () == IDOK)
	{
		dlg.GetCurrentFont (&lf);
		GetEditCtrl ()->m_clrText = dlg.m_cf.rgbColors;

		m_Font.DeleteObject ();
		m_Font.CreateFontIndirect(&lf);
		GetEditCtrl ()->SetFont (&m_Font);
	}
}

void CMDIEditorView::OnUpdateCursorPos(CCmdUI *pCmdUI)
{
	ASSERT_VALID (m_pEdit);

	CString str;
	str.Format(_T("Ln %i, Col %i"), m_pEdit->GetCurRow ()+1, m_pEdit->GetCurColumn ()+1);
	pCmdUI->SetText (str);

	pCmdUI->Enable (); 
}
