// CustomEditCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "Markers.h"
#include "CustomEditCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCustomEditCtrl

CCustomEditCtrl::CCustomEditCtrl()
{
	m_bEnableBreakpoints = FALSE;
	m_bFullLine = TRUE;
	m_bEnableLineMarker = TRUE;
	m_bDragLineMarker = FALSE;
}

CCustomEditCtrl::~CCustomEditCtrl()
{
}

BEGIN_MESSAGE_MAP(CCustomEditCtrl, CBCGPEditCtrl)
	//{{AFX_MSG_MAP(CCustomEditCtrl)
	ON_WM_CREATE()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CCustomEditCtrl message handlers

int CCustomEditCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CBCGPEditCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;

	return 0;
}

void CCustomEditCtrl::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	if (IsEnableBreakpoints() &&
		point.x < m_nLeftMarginWidth)
	{
		ToggleBreakpoint();
	}
	
	CBCGPEditCtrl::OnLButtonDblClk(nFlags, point);
}

void CCustomEditCtrl::OnLButtonDown(UINT nFlags, CPoint point)
{
	CBCGPEditCtrl::OnLButtonDown(nFlags, point);

	if (IsEnableLineMarker() &&
		point.x < m_nLeftMarginWidth)
	{
		CBCGPEditMarker* pMarker = NULL;
		if (GetMarker(GetCurRow(), &pMarker, g_dwLineMarkerType) != NULL)
		{
			m_bDragLineMarker = TRUE;
			MoveLineMarker();
		}
	}
}

void CCustomEditCtrl::OnLButtonUp(UINT nFlags, CPoint point)
{
	CBCGPEditCtrl::OnLButtonUp(nFlags, point);

	m_bDragLineMarker = FALSE;
}

void CCustomEditCtrl::OnMouseMove(UINT nFlags, CPoint point)
{
	CBCGPEditCtrl::OnMouseMove(nFlags, point);

	if (IsEnableLineMarker() && m_bDragLineMarker)
	{
		MoveLineMarker(-1, FALSE);
		RedrawMarkerArea ();
	}
}

void CCustomEditCtrl::OnDrawMarker (CDC* pDC, CRect rectMarker, const CBCGPEditMarker* pMarker)
{
	if (pMarker->m_dwMarkerType & g_dwBreakPointType)
	{
		CBrush br (RGB (127, 0, 0));
		CPen pen (PS_SOLID, 1, RGB (127, 0, 0));
		CBrush* pBrOld = pDC->SelectObject (&br);
		CPen* pOldPen = pDC->SelectObject (&pen);

		rectMarker.DeflateRect (1, 1);
		rectMarker.left = rectMarker.left + rectMarker.Width () / 2 - rectMarker.Height () / 2 - 1;
		rectMarker.right = rectMarker.left + rectMarker.Height ();
		pDC->Ellipse (rectMarker);
		pDC->SelectObject (pBrOld);
		pDC->SelectObject (pOldPen);
	}
	else
	{
		CBCGPEditCtrl::OnDrawMarker (pDC, rectMarker, pMarker);
	}

	if (pMarker->m_dwMarkerType& g_dwLineMarkerType)
	{
		CBrush br (RGB (255, 255, 0));
		CPen pen (PS_SOLID, 1, RGB (0, 0, 0));
		CBrush* pBrOld = pDC->SelectObject (&br);
		CPen* pOldPen = pDC->SelectObject (&pen);

		rectMarker.DeflateRect (1, 1);
		rectMarker.left = rectMarker.left + rectMarker.Width () / 2 - rectMarker.Height () / 2 - 1;
		rectMarker.right = rectMarker.left + rectMarker.Height ();
		LONG lXCenter = rectMarker.left + rectMarker.Width () / 2;
		POINT points[] = {
			{lXCenter, rectMarker.top}, 
			{rectMarker.right, rectMarker.top + rectMarker.Height () / 2}, 
			{lXCenter, rectMarker.bottom},
			{lXCenter, rectMarker.bottom - rectMarker.Height () / 4},
			{rectMarker.left, rectMarker.bottom - rectMarker.Height () / 4},
			{rectMarker.left, rectMarker.top + rectMarker.Height () / 4},
			{lXCenter, rectMarker.top + rectMarker.Height () / 4}};
		pDC->Polygon (points, 7);
		pDC->SelectObject (pBrOld);
		pDC->SelectObject (pOldPen);
	}
}

BOOL CCustomEditCtrl::EnableBreakpoints(BOOL bFl /* = TRUE */)
{
	const BOOL bEnableBreakpoints = m_bEnableBreakpoints;
	m_bEnableBreakpoints = bFl;

	return bEnableBreakpoints;
}

BOOL CCustomEditCtrl::ToggleBreakpoint(int nCurrRow /* = -1 */)
{
	if (nCurrRow == -1)
	{
		nCurrRow = GetCurRow();
	}

	BOOL bMarkerSet = ToggleMarker (nCurrRow, g_dwBreakPointType, NULL, FALSE, 2);

	if (bMarkerSet)
	{
		SetLineColorMarker (nCurrRow, RGB (255, 255, 255), 
			                RGB (127, 0, 0), TRUE, 0, g_dwColorBreakPointType, 2);
	}
	else
	{
		DeleteMarker (nCurrRow, g_dwColorBreakPointType);
	}

	return bMarkerSet;
}

void CCustomEditCtrl::RemoveAllBreakpoints()
{
	DeleteAllMarkers(g_dwBreakPointType | g_dwColorBreakPointType);
}

BOOL CCustomEditCtrl::ToggleStaticMarker(int nCurrRow /* = -1 */)
{
	if (nCurrRow == -1)
	{
		nCurrRow = GetCurRow();
	}
	CObList lstMarkers;
	GetMarkersInRange(nCurrRow, lstMarkers, g_dwColorHeaderType);
	BOOL bMarkerSet = !lstMarkers.IsEmpty ();

	if (!bMarkerSet)
	{
		SetLineColorMarker (nCurrRow, RGB (0, 0, 0), 
			                RGB (192, 220, 192), m_bFullLine, 3, g_dwColorHeaderType, 1);
		RedrawText (nCurrRow, nCurrRow + 3);
	}
	else
	{
		CBCGPEditMarker* pMarker = (CBCGPEditMarker*) lstMarkers.GetHead ();
		ASSERT_VALID (pMarker);
		int nRow = pMarker->m_nLine;
		DeleteMarker (nRow, g_dwColorHeaderType);
		RedrawText (nRow, nRow + 3);
	}

	return bMarkerSet;
}

BOOL CCustomEditCtrl::EnableFullLine(BOOL bFl /* = TRUE */)
{
	const BOOL bEnableFullLine = m_bFullLine;
	m_bFullLine = bFl;

	for (POSITION pos = m_lstMarkers.GetHeadPosition (); pos != NULL;)
	{
		CBCGPEditMarker* pMarkerNext = 
			DYNAMIC_DOWNCAST (CBCGPEditMarker, m_lstMarkers.GetNext (pos));

		if (pMarkerNext->m_dwMarkerType & g_dwColorStaticMarkerType ||
			pMarkerNext->m_dwMarkerType & g_dwColorHeaderType)
		{
			CBCGPLineColorMarker* pColorMarker = 
				DYNAMIC_DOWNCAST (CBCGPLineColorMarker, pMarkerNext);

			ASSERT_VALID (pColorMarker);
		
			pColorMarker->m_bFullLine = bFl;
			RedrawText (pColorMarker->m_nLine, 
				pColorMarker->m_nLine + pColorMarker->m_nLineCount);

		}
	}

	return bEnableFullLine;
}

BOOL CCustomEditCtrl::ToggleLineMarker(int nCurrRow /* = -1 */, BOOL bRedraw /* = TRUE */)
{
	if (nCurrRow == -1)
	{
		nCurrRow = GetCurRow();
	}

	return ToggleMarker (nCurrRow, g_dwLineMarkerType, NULL, bRedraw, 3);
}

void CCustomEditCtrl::RemoveLineMarker(BOOL bRedraw /* = TRUE */)
{
	DeleteAllMarkers (g_dwLineMarkerType, bRedraw);
}

BOOL CCustomEditCtrl::MoveLineMarker(int nCurrRow /* = -1 */, BOOL bRedraw /* = TRUE */)
{
	RemoveLineMarker (bRedraw);
	return ToggleLineMarker (nCurrRow, bRedraw);
}

BOOL CCustomEditCtrl::EnableLineMarker(BOOL bFl /* = TRUE */)
{
	const BOOL bEnableLineMarker = m_bEnableLineMarker;
	m_bEnableLineMarker = bFl;

	return bEnableLineMarker;
}
